/* mmf2c.c : extract first sequence chunk from SMAF file */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <netinet/in.h>

/* SMAF sequence data chunk ID */

#define SET_ID "Mtsu"
#define SEQ_ID "Mtsq"

#define ITEMPERLINE  12
#define ITEMPERLINE2 8

char aname[16];
char title[256];

int setup_flag = 0;

void options(int argc, char * const argv[])
{
  int optc;
  if (1 < argc) {
    while (-1 != (optc = getopt(argc, argv, 
                                "A:N:V:S:T:Ua:n:v:s:t:u"))){
      switch (optc) {
        case 'A': case 'a':
        case 'N': case 'n':
        case 'V': case 'v': // array name option
          strncpy(aname, optarg, sizeof(aname));
//          printf("-%c=\"%s\"\n", optc, optarg);
          break;
        case 'S': case 's': 
        case 'T': case 't': // song title option
          strncpy(title, optarg, sizeof(title));
//          printf("-%c=\"%s\"\n", optc, optarg);
          break;
        case 'U': case 'u': // setup chunk copy option
          setup_flag = 1;
          break;
        default:
          break;
      } // switch
    } // while
  } // if (0 < argc)
} // void options()

unsigned long int get_chunk(char *ID_str)
{
  int nxfer, i = 0;
  unsigned char b;
  unsigned long int d, chk_ID, j, chk_size;

  memcpy(&chk_ID, ID_str, 4);            // set chunk ID
  nxfer = fread(&d, 1, 4, stdin);        // get first 4 bytes
  while ((nxfer > 0) && (d != chk_ID)) { // loop until "Mtsu" found 
    d >>= 8;                             // shift one char 
    nxfer = fread((char *)(&d)+3, 
                   1, 1, stdin);         // read next char 
  }
  nxfer    = fread(&b, 1, 1, stdin); // get chunk size
  chk_size = b;
  nxfer    = fread(&b, 1, 1, stdin); // get chunk size
  chk_size = (chk_size << 8) + b;
  nxfer    = fread(&b, 1, 1, stdin); // get chunk size
  chk_size = (chk_size << 8) + b;
  nxfer    = fread(&b, 1, 1, stdin); // get chunk size
  chk_size = (chk_size << 8) + b;
  return (chk_size);
} // unsigned long int get_chunk()

int main(int argc, char * const argv[])
{
  int nxfer, i = 0;
  unsigned char b;
  unsigned long int d, chk_ID, j, chk_size;
// setup chunk
//  memcpy(&chk_ID, SET_ID, 4);            // set "Mtsu"
  chk_size = get_chunk(SET_ID);   // "Mtsu" for setup chunk
  memcpy(aname, SEQ_ID, sizeof(SEQ_ID)); //
  memset(title, 0, sizeof(title));
  options(argc, argv);
  j = 0;
  printf("// Song title: %s\r\xa\r\xa", title);
  printf("#include \"tonegen.h\"\r\xa");
  printf("\r\xa");
  printf("#ifdef SEQUENCER\r\xa");
  printf("\r\xa");
  if (setup_flag) { // setup chunk copy option
    printf("// Demo song setup data\r\xa");
    printf("// Song title: %s\r\xa\r\xa", title);
    printf("const unsigned char %s_su[] = {", aname);
    nxfer = fread(&b, 1, 1, stdin); // get data
    do {
      printf("%s", ( (0 == i) ? "\r\xa  " : "") );
      printf("0x%.2x", b);
      i = (++i % ITEMPERLINE2);
      j += 1; // advance 
      if (j < chk_size) printf(", ");
    } while ((j < chk_size) && (0 != fread(&b, 1, 1, stdin))); /* do */
    if (0 == i) printf("\n  ");
    printf("\r\xa}; // const unsigned char %s_su[]\r\xa", aname);
    printf("\r\xa");
    printf("\r\xa");
  } // if (setup_flag) ...
// sequence chunk
  i = 0;
  memcpy(&chk_ID, SEQ_ID, 4);            // set "Mtsq"
  chk_size = get_chunk(SEQ_ID);   // "Mtsq" for sequence chunk
//  memcpy(aname, SEQ_ID, sizeof(SEQ_ID)); //
//  memset(title, 0, sizeof(title));
//  options(argc, argv);
/*
  nxfer = fread(&d, 1, 4, stdin);        // get first 4 bytes
  while ((nxfer > 0) && (d != chk_ID)) { // loop until "Mtsq" found 
    d >>= 8;                             // shift one char 
    nxfer = fread((char *)(&d)+3, 
                   1, 1, stdin);         // read next char 
  }
*/
  printf("// Demo song sequence data\r\xa");
  printf("// Song title: %s\r\xa\r\xa", title);
  printf("const prog_uint8_t %s[] = {", aname);
  for (j = 0; j < 4; j++) { // output chunk ID
    printf("%s", ( (0 == i) ? "\r\xa  " : "") );
    printf("0x%.2x, ", 0xff & (chk_ID) );
    chk_ID >>= 8; // next char 
    i = (++i % ITEMPERLINE);
  }
  d = chk_size;
  for (j = 0; j < 4; j++) { // output chunk size
    printf("%s", ( (0 == i) ? "\r\xa  " : "") );
    printf("0x%.2x, ", 0xff & (d >> 24) );
    d <<= 8; // next char 
    i = (++i % ITEMPERLINE);
  }
  for (j = 0; j < chk_size; j++) { // output chunk data
    fread(&d, 1, 1, stdin);
    printf("%s", ( (0 == i) ? "\r\xa  " : "") );
    printf("0x%.2x, ", 0xff & d );
    i = (++i % ITEMPERLINE);
  } // for (j 
  for (j = chk_size; j < ((~0x03) & (chk_size+3)); j++) {
    printf("%s", ( (0 == i) ? "\r\xa  " : "") );
    printf("0x%.2x, ", 0);
    i = (++i % ITEMPERLINE);
  } // for (j 
  if (0 == i) printf("\r\xa  ");
  printf("\r\xa}; // const prog_uint8_t %s[]\r\xa", 
          aname);
  printf("#endif\r\xa");
  return(0);
} /* main() */
