/**************************************************/
/* Envelope generator                             */
/*                                                */
/* NON COMMERCIAL USE ONLY                        */
/*                                                */
/* 2007/06/26 Rev. 1.01 by pcm1723                */
/*                                                */
/**************************************************/

#include "tonegen.h"

int16_t eg(unit_prop_t *u_p, op_prop_t *o_p) 
{                                                                               
  int d, s, eg_underf; 
//  int egr;
  uint32_t egsum, egam, lin_level;

  d  = (o_p->damp_q << 1);   // shift last damper state
  d |= (0x01 & u_p->damper); // current damper state
  d &=  0x03;               // mask off
  s  = (o_p->gate_q << 1);  // shift last gate state
  s |= (0x01 & u_p->gate);  // current gate state
  s &=  0x03;               // mask off
  o_p->gate_q  =  s;        // save it
  d |= s;                   // combine GATE and DAMPER
  o_p->damp_q  =  d;        // save it
  if (0x01 == s) {// detect pos edge of gate, start ATTACK
    o_p->egmode = ATTACK;
    o_p->attacc = 0;  // clear EG attack accumulator
//    o_p->egacc  = 0;  // clear EG accumulator (start from 0 dB)
    o_p->sl_lb  = (o_p->vp.sl << 16); // sustain level
    o_p->ol_lin = 0;
    o_p->egrate =  o_p->vp.ar;
    if (o_p->egrate) { // add rate offset
      o_p->egrate += o_p->rof;
      if (60 <= o_p->egrate) o_p->egrate = 60; // clip to 60
      o_p->egrate += 13; // attack offset
    } // if 
    o_p->eginc  = eg_tab[o_p->egrate];
    o_p->phacc  = 0; // start from 0
    o_p->opout  = 0; // operator output
/*
lcd_gotoxy(0,o_p->op_num);
printf("N%1.1x r=%2.2x o=%2.2x ", o_p->op_num, o_p->egrate, o_p->rof);
*/
} // if (0x01 == s)
  else if ((0x02 == d) /*& (o_p->vp.egt)*/) { // detect neg edge of damper, start RELEASE
    o_p->egrate = o_p->vp.rr;
    if (o_p->egrate) { // add rate offset
      o_p->egrate += o_p->rof;
      if (60 <= o_p->egrate) o_p->egrate = 60; // clip to 60
    } // if 
    o_p->last_egm = o_p->egmode; // remember current EG mode
    o_p->egmode = RELEASE; // release mode
    o_p->eginc  = eg_tab[o_p->egrate];
//    if (1 == o_p->op_num) u_p->keyoff = 0;
  } // else if ((0x02 == o_p->gate_q ...
  if (o_p->egmode) { // operator active
    egam  = (o_p->tl_lb + o_p->vol_lb + o_p->am_lb);
    if (ATTACK == o_p->egmode) o_p->attacc += o_p->eginc;
    else {
      o_p->egacc  += o_p->eginc;
      egsum = (o_p->egacc + egam);
      eg_underf = (EG_96dB <= o_p->egacc);   // EG underflow flag (att. > 96dB)
      if (eg_underf) { // EG att. > 96dB ?
        lin_level = 0; // mute output
      } // if (eg_underf) ...
      else { // EG still operate
        lin_level = LB_to_LIN(egsum >> 16); // convert to linear
      } // if (eg_underf) ... else ...
    } // if (ATTACK ... else ...
    switch (o_p->egmode) { //
      case ATTACK :
//        if (eg_underf) { // trip point to decay
//        if (EG_48dB <= o_p->egacc) { // trip point to decay
        if (u_p->voice_param_change) {
          copy_voice(u_p); // update voice data
        } // if (u_p->voice_param_change);
        if (EG_60dB <= o_p->attacc) { // trip point to DECAY
          o_p->egmode = DECAY;
          o_p->egrate = o_p->vp.dr;
          if (o_p->egrate) { // add rate offset
            o_p->egrate += o_p->rof;
            if (60 <= o_p->egrate) o_p->egrate = 60; // clip to 60
          } // if 
          o_p->eginc  = eg_tab[o_p->egrate];
          o_p->egacc  = 0;
          lin_level   = LB_to_LIN(egam >> 16);
/*
lcd_gotoxy(0,o_p->op_num);
printf("N%1.1x egam=%8.8x ", o_p->op_num, egam); 
*/
        } // if (eg_underf) ...
        else { // still in attack
//          lin_level = LB_to_LIN((EG_96dB-o_p->egacc+egam) >> 16);
//          lin_level = (LB_to_LIN(egam >> 16) - lin_level);
//          lin_level = ((LB_to_LIN(egam >> 16) >> 16) * (o_p->egacc >> 11))<<1;
          o_p->egacc = (LB_to_LIN_s((o_p->attacc >> 16), 11)*3)>>2;
          egsum = egam + o_p->egacc;
          lin_level = LB_to_LIN(egsum >> 16);
        }
        break;
      case DECAY:
        if (o_p->sl_lb <= o_p->egacc) { // crossing sustain level
          o_p->egacc = o_p->sl_lb; // replace by sustain level
          egsum = (o_p->egacc + egam); // re-calculate EG level
          lin_level = LB_to_LIN(egsum >> 16); // convert to linear
          if (o_p->vp.egt) { // EGT==1, sustain type
            o_p->egmode = SUSTAIN;
            o_p->egrate = 0; // never change EG level until release
            o_p->eginc  = 0;
/*
lcd_gotoxy(0,o_p->op_num);
printf("N%1.1x r=%2.2x m=%2.2x ", o_p->op_num, o_p->egrate, o_p->egmode);
*/
          } // if (o_p->egt) ...
          else { // EGT==0, percussive type, go to release
            o_p->egmode = RELEASE;
            o_p->egrate = o_p->vp.rr; // release rate
            if (o_p->egrate) { // add rate offset
              o_p->egrate += o_p->rof;
              if (60 <= o_p->egrate) o_p->egrate = 60; // clip to 60
            } // if (o_p->egrate)
            o_p->eginc  = eg_tab[o_p->egrate];
          } // else ...
        } // if (o_p->sl_lb ...
        else if (eg_underf) { // trip point to operator off
          o_p->egmode = OFF; // EG off
          lin_level = 0;  // mute output
        } // else if (eg_underf) ... else ...       
        break;
      case FORCE_DAMP:     // check for force damp
      case FORCE_RELEASE : // check for force release
        o_p->egrate = o_p->vp.rr;
        if (FORCE_DAMP == o_p->egmode) { // if force damp, replace by FORCE_RELEASE_RATE
          o_p->egrate = FORCE_RELEASE_RATE;
        }
        if (o_p->egrate) { // add rate offset
          o_p->egrate += o_p->rof;
          if (60 <= o_p->egrate) o_p->egrate = 60; // clip to 60
        } // if 
        o_p->egmode = RELEASE; // release mode
        o_p->last_egm = o_p->egmode; // remember current EG mode
        o_p->eginc  = eg_tab[o_p->egrate];
        break;
      case RELEASE : // check for re-damper
//        o_p->attacc = 0;
        if (eg_underf) { // EG att. > 96dB ?
           o_p->egmode = OFF; // EG off
           lin_level = 0; // mute output
        } // if (eg_underf)
        else if (0x01 & d) { // Damper pedal pressed at release state
          if (o_p->vp.egt) { // sustain type
            o_p->egmode = o_p->last_egm;
            o_p->egrate = (6 << 2);
            o_p->eginc  = eg_tab[o_p->egrate];
          } // if(o_->egt)
        } // else if (0x01 ...
        break;
      default:
        if (eg_underf) o_p->egmode = OFF; // operator OFF
        break;
    } // switch (o_p->egmode)
  } // if (o_p->egmode) ...
  else { // operator off
    lin_level = 0; // operator off
    if (u_p->voice_param_change) {
      copy_voice(u_p);
    }
  } // if (o_p->egmode) ... else ...
  o_p->ol_lin = lin_level;
#if syn_CV
      if (syn_CV_ch == u_p->midi_ch) {
        if (1 == o_p->vp.op_num) { // operator 1, carrier
//          DAC2DAT = o_p->ol_lin >> (16-12);
        }
        else {                  // operator 0, modulator
//          DAC1DAT = o_p->ol_lin >> (16-12);
        }
      } // if (0 == u_p->midi_ch)
      if (syn_CV_ch == u_p->midi_ch) {
        if (1 == o_p->vp.op_num) { // operator 1, carrier
          DA0CS1 = o_p->ol_lin >> 24;
        }
      } // if (0 == u_p->midi_ch)
#endif
  return(0);
}

void set_egmode(unit_prop_t *u_p, int egm)
{
  int i;
  for (i = 0; i < 2; i++) { // for each operator
    op_prop_t *o_p = &(u_p->op_prop[i]);
    o_p->egmode = egm;   
    o_p->gate_q = 0;
    o_p->damp_q = 0; 
  } // for
} // void set_egmode()

/*
lcd_gotoxy(0,o_p->op_num);
printf("N%2.2x o=%8.8x ", 
       u_p->midi_note, o_p->ol_lin);
*/
