/*********************************************************/
/*                                                       */
/* lcd_arm.c :  Hardware dependent LCD support routines  */
/*                                                       */
/* NON COMMERCIAL USE ONLY                               */
/*                                                       */
/* 2007/03/26 Rev. 1.00 by pcm1723                       */
/*                                                       */
/*********************************************************/

#include "tonegen.h"

#ifdef LCD

uint8_t sw_bits;

void LCD_DATA_OUT(uint32_t d)
{
   LCD_DATA_PORT = (LCD_DATA_PORT & 0xffc00000)  // preserve P4.7 .. P4.6
          | ((0x00f000 & d) <<  4) // output high nibble (D7..D4) to P4.3 .. P4.0
          | (((LCD_EBIT | LCD_RSBIT) & d) << 16); // output E / RS bit
} // void LCD_DATA_OUT()


/* negate LCD E signal */
void lcd_E_down(void)
{
  LCD_DATA_DDR  |=  (LCD_DATA_BITMASK << 24);  // set LCD_data/SW port to output
  LCD_DATA_PORT &= ~(LCD_EBIT << 16);            // negate E signal
  LCD_DATA_DDR  &= ~(LCD_DATA_BITMASK << 24);    // set LCD_data/SW port to input
} // void lcd_E_down()


void out_LCD(int d, int s)
{
  sw_bits  = SW_BITMASK & LCD_DATA_PIN;  // read SW port
  sw_bits &=  (0xfe ^ GET_PANEL_KEY);    // combine with onboard SW2
  d = (0xff & d) | (0xf000 & ( s ? d << 4 : d)); // low nibble if (1 == s)
  LCD_DATA_OUT(d );           // write data to LCD data port
  LCD_DATA_OUT(d | LCD_EBIT); // write data to LCD data port and E trigger
  LCD_DATA_DDR  |=  (LCD_DATA_BITMASK << 24);  // set LCD_data/SW port to output
} // void out_LCD()

/* display CPU load */
void lcd_cpuload(void)
{
  int i, d;
  d = (ADCDAT >> 16); // get ADC data in b11..b0, (0.0 <= d < 1.0) at Q12 format
  lcd_gotoxy(13, 0);  // set position
  for (i = 0; i < 2; i++) { // convert to decimal two digit
    d *= 10; // multiply by 10, (0.0 <= d < 10.0) at Q12 format
    lcd_putchar(hex_str[0x0f & (d >> 12)]); // extract b15..b12 and convert to ASCII
    d &= 0x0fff; // remove integer part of Q12 format
  } // for
} // void lcd_cpuload()


void wait_ms(int ms)
{
  int i, j;
  for (i = 0; i < ms; i++) {
    for (j = 0; j < SEQTICK1ms; j++) {
      while (0 == t1_tick) ;
      t1_tick = 0;
    } // for (j
  } // for (i
} // void wait_ms()

/* write high nibble (4bit config) or write full 8bit (8bit config) */
void lcd_write4(int d)
{
  LCD_DATA_OUT(LCD_EBIT | d | ((0x8000 & d) >> 9)); // write data to LCD data port and raise E trigger
  wait_ms(1);              // wait for 1ms
  lcd_E_down();            // down E trigger
  wait_ms(1);              // wait for 1ms
} // void lcd_write4()

/* for 4bit configuration, write high nibble first, then write low nibble */
void lcd_write8(int d)
{
  lcd_write4(d); //  write full 8bit (8bit config), or write high nibble (4bit config)
#ifdef LCD_4bit
  LCD_DATA_OUT(LCD_EBIT | ((0x0800 & d) >> 5)
            | (0xf000 & (d << 4)) | (0xff & d)); // write low nibble to LCD data port and raise E trigger
  wait_ms(1);              // wait for 1ms
  lcd_E_down();            // down E trigger
  wait_ms(1);              // wait for 1ms
#endif
} // void lcd_write8()



#endif
