/**************************************************/
/*                                                */
/* smafdec.c : SMAF sequence chunk decoder        */
/*                                                */
/* NON COMMERCIAL USE ONLY                        */
/*                                                */
/* 2007/03/26 Rev. 1.00 by pcm1723                */
/*                                                */
/**************************************************/

#include "tonegen.h"

#ifdef SEQUENCER
seq_status_t seq_st[MAXSEQCH];

int seq_p;

// 2-byte expresson code to MIDI expression 
unsigned char exptab[16] = {
  0x00, 0x00, 0x1f, 0x27, 0x2f, 0x37, 0x3f, 0x47, 
  0x4f, 0x57, 0x5f, 0x67, 0x6f, 0x77, 0x7f, 0x7f
}; // unsigned char exptab[]

// get duration, returns decoded duration value (0..16511) 

int get_dur(const uint8_t *sq)
{
  int dur;
  dur = sq[seq_p++]; /* get first byte */
  if (0x80 & dur) {    /* if S=1, 2 byte data */
    dur = ((0x7f & dur) << 7) + sq[seq_p++] + 128; 
  }
  return(dur); 
} // int get_dur() 

// get event, returns 1 if EOS found, 0 otherwise 

int get_event(const uint8_t *sq)
{
//  int i;
  int ev0, ev1, ev2, ety, ch, oct, note, typval;
  ev0 = sq[seq_p++]; // byte 0 of event
  switch (ev0) { // decode event
    case E1_NOP :  // NOP or SysEx event
      ev1 = sq[seq_p++]; // byte 1 of event
      break;
    case E1_MISCEV : { // other events but note event 
      ev1 = sq[seq_p++]; // byte 1 of event 
      if (0 == ev1) return(1); // EOS found 
      ch = (ev1 >> 6);         // b7b6:      channel 
      typval = (0x0f & ev1);   // b3b2b1b0 : type/val 
      ety = (0x30 & ev1) >> 4; // b5b4 : event type 
      switch (ety) {
        case E2_EXPRESS :  // 2-byte expression 
         if (MAXMIDICH > ch) {
           ch_prop_t *c_p = &(ch_prop[ch]);
           c_p->MD_expression = exptab[typval];
           calc_vol_exp(c_p);
          } // if
          break;
        case E2_PCHBEND :  // 2-byte pitch bend 
          break;
        case E2_MODURAT :  // 2-byte moduration
          break;
        default:  // 3-byte event 
          ev2 = sq[seq_p++]; // get value byte
          switch (typval) {
            case E3_PGMCHNG : { // program change
              if (MAXMIDICH > ch) {
//                ch_prop[MD_ch].MD_progno = ev2;
//                setup_voice(ch, ev2);
              }
              break;
            case E3_BANKSEL :  // bank select
              break;
            case E3_OCTSHFT :  // octave shift
              oct = ev2;
              if (0x80 & ev2) oct = 0x80 - oct;
              seq_st[ch].octshift = oct;
              break;
            case E3_MODURAT :  // moduration 
              break;
            case E3_PCHBEND :  // pitch bend
              break;
            case E3_VOLUME :  // volume
              if (MAXMIDICH > ch) {
                ch_prop_t *c_p = &(ch_prop[ch]);
                c_p->MD_volume = ev2;
                calc_vol_exp(c_p);
              } // if
              break;
            case E3_PAN :  // pan 
              break;
            case E3_EXPRESS :  // expression
              if (MAXMIDICH > ch) {
                ch_prop_t *c_p = &(ch_prop[ch]);
                c_p->MD_expression = ev2;
                calc_vol_exp(c_p);
              } // if
              break;
            default:;
          } // switch(typval)
          break;
        } // 3-byte event 
      } // switch (ety)
      break;
    } /* case E1_MISCEV */
    default:  // note event
      ch  = (ev0 >> 6);        // b7b6: channel 
      oct = (0x30 & ev0) >> 4; // b5b4: octave 
      seq_st[ch].gatetime = get_dur(sq);
      note = (0x0f & ev0);     // b3b2b1b0: note number
      note += 12*(oct + seq_st[ch].octshift + 3);
      seq_st[ch].notenumber = note; // remember note for keyoff
      if (MAXMIDICH > ch) {
        ch_prop_t *c_p = &(ch_prop[ch]);
        midi_note_on(c_p, note, 0x7f); // MIDI note on with velocity = 127
      } // if
      break;      
  } // switch (ev0)
  return(0);
} // int get_event()
#endif
