/**************************************************/
/*                                                */
/* tonegen.h : STARM tone generator header file   */
/*                                                */
/* NON COMMERCIAL USE ONLY                        */
/*                                                */
/* 2010/09/19 Modified for TGSTCQ/TGSTBEE/TGSTBM  */
/* 2008/04/19 Rev. 1.01 by pcm1723                */
/*                                                */
/**************************************************/

#ifndef TONEGEN_H
#define TONEGEN_H

//#define MIDIONLY
//#define DEMOONLY

// ADI ADuC7026 definition
#ifdef __THUMB__
#include <aduc7026.h>          // ADuC7026 register definition 
#endif

// STmicro STM32F103 definition
#ifdef __thumb__
 #ifndef ADUC7026 
  #if defined(STM32F10X_MD_VL)
    #include <stm32f10x.h>
    #ifndef STM32VL
      #define STM32VL
    #endif
    #define HAS_12BIT_DAC 1
  #else
    #include "stm32f10x_lib.h"  // STM32F10x Library Definitions
  #endif
  #define FORCE_FAST
  #ifndef STM32F103
   #define STM32F103
  #endif
 #endif
#endif

#if defined(USE_STDPERIPH_DRIVER)
  #undef USE_STDPERIPH_DRIVER
#endif

#ifdef __TID__ // IAR compiler 
  #if (6 == (0x0f & (__TID__ >> 4))) // Cortex-M3 processor 
    #include <ST\iostm32f10x.h>  // STM32F10x IO Definitions
    #include <intrinsics.h>
    #define FORCE_FAST
    #ifndef  STM32F103
     #define STM32F103
    #endif
    #ifndef  __IAR__
     #define __IAR__
    #endif
  #endif
#endif


#ifdef __v850__
  #pragma ioreg
  #include "inttypes.h"
#else
  #include <inttypes.h>
#endif

#include <stddef.h>

typedef unsigned int uint;

#define MAXSHORT 32767

#define MAX_CONFIG 5

#define MAXSEQCH      16
#define ABS_MAX_UNITS 16
#define ABS_MAX_PARTS 16

#define USE_DACBUF 1

#define USEWAVEQUE 1
#define WAVEQUESIZE 64

//#define debug 0

//#define REVERB 0

#ifndef MIDIONLY
#define SEQUENCER
#endif

//#define LCD 1

#define LCD_4bit 1

#define EXTRADEMO 1

#define DEBOUNCE_ms (14)

// PRESS/CLICK decision threshold (29 * 14 ms = 406 ms) 
#define SW_RL_TH (29)

/* Support for analog synthesizer Control Voltage output */
//#define syn_CV 1

/* Binary point pf phase accumulator is at right of b28 */
#define Q_PHACC  (28)

/* Binary point pf f-number is at right of b15 */
#define Q_FNUMBER (15)

/* Lowest frequency of MIDI note (key no = 0) */ 
#define MIN_MIDI_FREQ (8.1757090)

/* log2 of lowest MIDI frequency */
#define LB_MIN_MIDI_FREQ (3.031344)

#define FORCE_RELEASE_RATE (4*14)

// placeholder for AVR specific program memory access
#define READ_BYTE(x)  (x)
#define READ_WORD(x)  (x)
#define READ_DWORD(x) (x)
#define PROGMEM

#define prog_uint8_t  uint8_t
#define prog_sint8_t  sint8_t
#define prog_int8_t    int8_t
#define prog_uint16_t uint16_t
#define prog_sint16_t sint16_t
#define prog_int16_t   int16_t
#define prog_uint32_t uint32_t
#define prog_sint32_t sint32_t
#define prog_int32_t   int32_t



extern int MAXMIDICH;
extern int MAXUNITS;
extern int SEQTICK1ms;
extern int SEQTICK4ms;
extern int FS_ASC;

enum panel_mode_t {NORMAL, MIDI_MON, CH_ASSIGN, SHOW_NRPN, SHOW_Sys_Ex, SET_CONFIG, SYS_RESET};

enum sw_value_t {NOCHANGE, CLICK, PRESS};

enum egmode_t {OFF, ATTACK, DECAY, SUSTAIN, RELEASE, REDAMPER, FORCE_RELEASE, FORCE_DAMP};


#ifdef LCD
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif


/* Buffer size of MIDI System Exclusive message */
#define MAXSYSEXLEN 40

/* F-number table entry size (12 semitone x 128 step = 1536) */
#define FNUMTAB_MAX 1536

/* Envelope Generator table size */
#define EG_TAB_SIZE 79

/* Sine table size */
//#define STAB_SIZE 4097
#define STAB_SIZE 8192

extern uint seq_duration;
extern int  seq_tick;
extern int  tick1ms;
extern int  t4ms0;
extern int  demoflag;
extern int  loopflag;
extern int  demo_no;
extern int  demosongs;
extern int  basech;
extern int  eos;
extern int  panel_mode;
extern int  last_note;
extern int  midi_mon_part;
extern int sys_mode_index;
extern int debounce;
extern uint32_t panel_sw_q;
extern int8_t serial_midi;
extern int8_t midithru; 
extern uint16_t adc_data;
extern int     delay_mem_size;
extern int16_t delay_ms;

#define MAX_RPN_FUNCS  6
#define MAX_NRPN_FUNCS 3

extern const int   LFO_inc_tab[4];
extern const int   mult_tab[16];
extern const int16_t stab[STAB_SIZE];
extern uint8_t MS_run_stat, MD_2nd, MD_3rd, MD_data, MD_ch;
extern uint8_t mon_ch, syn_CV_ch;
extern int   MD_bcnt;
extern int   sysx_cnt;
extern int   demoflag, loopflag;
extern int   note_ofs;
extern int   oct_ofs;
extern int8_t ch_to_part[16];
extern int8_t part_to_ch[ABS_MAX_PARTS];
extern volatile int   t1_tick;
extern int  fastchip;

#ifdef USEWAVEQUE
extern volatile int   wq_wr_ix, wq_rd_ix;
extern int16_t wave_que[WAVEQUESIZE];
#endif

#ifdef syn_CV
#ifdef __v850__
#define SPICLK_kHz 250
#define SPIQUESIZE 16
extern volatile int spiq_wr_ix, spiq_rd_ix;
extern int8_t spi_que[SPIQUESIZE];
#define CHECK_SPIQ   (spiq_wr_ix != spiq_rd_ix)
#define PUT_SPIQ(x)  (spi_que[spiq_wr_ix++] = (x), spiq_wr_ix &= (SPIQUESIZE-1))
#define XMIT_SPI     (CB3TX = spi_que[spiq_rd_ix++], spiq_rd_ix &= (SPIQUESIZE-1))
#define SPI_1ST_BYTE (0 == (0x01 & spiq_rd_ix))
#define SS_HIGH      (P9H.2 = 0x01)
#define SS_LOW       (P9H.2 = 0x00)
#define GATE_ON      (P9H.5 = 0x01)
#define GATE_OFF     (P9H.5 = 0x00)
#define syn_CV_ofs   (152)
#endif
#endif


extern char banner_str[] ;

#ifdef LCD
#define LCD_ROWS  2
#define LCD_COLS 16
#define LCD_BUFSIZE (LCD_ROWS*LCD_COLS)
#define CGRAMSIZE 64
// time constant for 3 sec in terms of 4 ms tick
#define LCD_DISP_TICKS 750
#define LCD_BLINK_ms   500
#define LCD_BLINK_CHAR 0xa5

#if defined(__v850__)
  #define LCD_EBIT  0x40
  #define LCD_RSBIT 0x10
  #define LCD_DATA_PORT     P3L
  #define LCD_DATA_DDR      PM3L
  #define LCD_DATA_PIN      P3L
  #define LCD_DATA_BITMASK  0x78
  #define LCD_DATA_SFT      1

  #define LCD_CTRL_PORT     P0
  #define LCD_CTRL_DDR      PM0

  #define ADC_DATA_PORT     ADA0CR11
  #define ADC_BITPOS        0
  #define ADC_BITMASK       0xff


  #define SW_BITMASK        LCD_DATA_BITMASK
  #define SW_BIT_SHIFT      3
  #define SW_SENS_PORT      LCD_DATA_PORT
  #define SW_SENS_DDR       LCD_DATA_DDR
  #define SW_SENS_PIN       LCD_DATA_PIN
  #define SW_SENS_BITMASK   LCD_DATA_BITMASK

  #define SW_RE_BITMASK     0x03
  #define SW_RE_DETENT      0x03
  #define SW_RE_FWD         0x02
  #define SW_RE_BACK        0x01

  #define SMIDI_SENS_PORT    LCD_DATA_PIN
  #define SMIDI_SENS_BITMASK 0x40

  #define LCD_DATA_PORT_OUT(port,mask,x,s) (port = (~mask & port) | (mask & ((x)>>s)))

  #define LCD_DATA_OUT(x) (LCD_DATA_PORT_OUT(LCD_DATA_PORT,LCD_DATA_BITMASK,x,LCD_DATA_SFT))

  #define LCD_CTRL_OUT(e,rs) (LCD_CTRL_PORT = (~(LCD_EBIT | LCD_RSBIT) & LCD_CTRL_PORT) | (LCD_EBIT & (e)) | (LCD_RSBIT & (rs)))
  #define LCD_E_OUT(e) (LCD_CTRL_PORT = (LCD_EBIT & e) | (~LCD_EBIT & LCD_CTRL_PORT))
#endif

extern const char hex_str[17];
extern void lcd_puthex1(int i);
extern void lcd_puthex2(int i);
extern int  lcd_putchar(int c);
extern void lcd_putstr(char* s);
extern void lcd_draw_panel(int panel_no);
extern const uint8_t cgram[CGRAMSIZE];
extern uint8_t lcd_buf[LCD_BUFSIZE];
extern uint8_t lcd_buf2[LCD_BUFSIZE];
extern uint8_t *lcd_buf_p;
extern uint8_t *lcd_buf_w;
extern uint8_t lcd_row, lcd_col, lcd_buf_ind;
extern uint8_t lcd_disp_row, lcd_disp_col, lcd_cur_pos;
extern int16_t lcd_blink_timer;
extern uint8_t lcd_blink_flag;
extern int16_t lcd_disp_timer;
extern void  lcd_write4(int d);
extern void  lcd_write8(int d);
extern void  lcd_refresh(uint8_t s);
extern void  lcd_init();
extern void  lcd_clear(uint8_t* lcd_buf);
extern void  lcd_gotoxy(int x, int y);
extern void wait_ms(int ms);
extern void out_LCD(int d, int s);
extern void lcd_cpuload(void);
extern void lcd_E_down(void);
extern void lcd_dbg_hex1(uint8_t pos, int8_t n);
extern void lcd_dbg_hex2(uint8_t pos, int16_t n);
extern void lcd_dbg_hex4(uint8_t pos, int16_t n);
extern void lcd_dbg_hex8(uint8_t pos, int32_t n);
extern void lcd_lba_hex1(uint8_t pos, int8_t n);
extern void lcd_lba_hex2(uint8_t pos, int8_t n);

#define MON_CH_DISP_OFS  (0)
#define MON_SW_DISP_OFS  (1)
#define MON_PC_DISP_OFS  (MON_CH_DISP_OFS+6)
#define MON_CC_DISP_OFS  (MON_CH_DISP_OFS+11)

#define MON_KEY_DISP_OFS (LCD_COLS+2)
#define MON_OCT_DISP_OFS (2)
#define MON_NN_DISP_OFS  (MON_KEY_DISP_OFS-2)
#define MON_VEL_DISP_OFS  (MON_KEY_DISP_OFS+12)

#define MON_RPN_DISP_OFS (6)
#define MON_RPN_MSB_OFS  (11)
extern uint8_t sw_bits;
#endif

extern uint32_t inc_sw_q, dec_sw_q;
extern uint16_t inc_sw_rl, dec_sw_rl;
extern int8_t   sw_val;
extern int8_t   re_state;

extern int8_t  sw_input(void);
extern void panel_cmd(int8_t sw);

extern const int16_t lb2lin[64];

extern const int16_t mv2lb[128];
extern const int32_t  eg_tab[79];
extern void set_config(int mode);
extern void enable_interrupt(void);
extern void seq(void);
extern void demo_off(void);
extern void	plaInitialize(void);
extern void mididec(int data);
extern void init_seq(void);
extern void seq_next_song(void);
extern void seq_songsel(int song_no);
extern const uint8_t  ks60tab[12];
extern const uint32_t portab[128];
extern const uint16_t fnum128[FNUMTAB_MAX];
extern const uint8_t  octkey[128];

#pragma pack(2)
struct ma2reg {
  uint8_t KSR : 1; // Key Scaling for Rate
  uint8_t SUS : 1; // SUStain flag
  uint8_t EGT : 1; // EG Type flag
  uint8_t VIB : 1; // VIBrato flag
  uint8_t MULT: 4; // freq MULTiplier
  uint8_t DR  : 4; // Decay   Rate
  uint8_t RR  : 4; // Release Rate
  uint8_t SL  : 4; // Sustain Level
  uint8_t AR  : 4; // Attack  Rate
  uint8_t KSL : 2; // Key Scaling for Level
  uint8_t TL  : 6; // Total   Level
  uint8_t WS  : 3; // Waveform Select
  uint8_t AM  : 1; // Amplitude Modulation flag
  uint8_t DAM : 2; // Depth of Amplitude Modulation
  uint8_t DVB : 2; // Depth of ViBrato
#if (defined(STM32F103) | defined(STM32F10X_MD_VL))
  uint8_t pad0; // padding for 2-byte alignment
#endif
}; // struct ma2reg

struct voicedata {
  uint8_t pad0;    // padding for alignment
  uint8_t bank;    // bank LSB
  uint8_t pcno;    // program change number
  uint8_t ALG : 3; // ALGorithm
  uint8_t FB  : 3; // FeedBack level
  uint8_t LFO : 2; // Low Frequency Osccillator
  struct ma2reg op[2];   // MA2 operator data
}; // struct voicedata

/* MA-2 2-operator voice data buffer */
/*
struct ma2vdat {
  uint8_t d[16];
};
*/ 

typedef uint8_t ma2vdat[16];

#pragma pack()

//extern const union ma2voice ma2v[128];

//extern const struct ma2vdat vma2[128];
//#ifdef __IAR__
//extern volatile __no_init const ma2vdat vma2[128] @ 0x08010000;
//#else
extern const ma2vdat vma2[128];
//#endif
/* operator decoded voice parameter structure */

typedef struct op_vparam_st {
  uint8_t ksr;     // key scaling rate  (voice data)
  uint8_t egt;     // EG type           (voice data)
  uint8_t vib;     // vibrato flag      (voice data)
  uint8_t mult;    // freq multiplier (linear 1..30) 
  uint8_t dr;      // EG delay  rate    (voice data)
  uint8_t rr;      // EG release rate   (voice data)
  uint16_t sl;      // EG sustain level  (voice data)   
  uint8_t ar;      // EG attack rate    (voice data)          
  uint8_t ksl;     // key scaling level (voice data)
  uint16_t tl;      // EG total   level  (voice data)           
  uint8_t wavesel; // wave select       (voice data)          
  uint8_t am;      // AM flag           (voice data)
  uint8_t dam;     // AM depth          (voice data)
  uint8_t dvb;     // vibrato depth     (voice data)
  uint8_t carrf;   // carrier flag 
  uint8_t op_num;  // Operator number
  int16_t (*op_func_p)(void *o_p);
} op_vparam_t, *op_vparam_p;

/* unit voice parameter */

typedef struct unit_vparam_st {
    uint8_t fb;        // modulator feedback depth       
    uint8_t lfo;       // LFO setting                    
    uint8_t alg;       // algorithm  
} unit_vparam_t, *unit_vparam_p;

/* operator property definition */
typedef struct {
//    int16_t  (*op_func_p)(void *op_p); // operator function pointer
    uint32_t  phacc;   // phase accumulator 
    uint32_t  ph_inc;  // phase increment (pitch) 
    int32_t   mod_in;  // phase modulation input (modulator or feedback)
    uint32_t   egacc;   // EG accumulator           
    uint32_t   attacc;  // EG attack accumulator           
    int32_t   eginc;   // EG acc increment        
    int32_t   eginit;  // EG acc init value        
    int32_t   ol_lin;  // current out level in lin 
    int32_t   tl_lin;  // total level in linear    
    int32_t   sl_lin;  // sustail level in linear  
    int32_t   tl_lb;   // TL + KSL + VEL in log2      
    int32_t   sl_lb;   // sustain level in log2      
    int32_t   vol_lb;  // VOL + EXP in log2      
    int32_t   am_lb;   // AM in log2      
    int16_t   egrate;  // EG rate                  
    int16_t   opout;   // wave output of this op  
    op_vparam_t vp; // voice parameters
/*
    int    mult;    // freq multiplier (linear 1..30) 
    int    tl;      // EG total   level  (voice data)           
    int    sl;      // EG sustain level  (voice data)   
    uint8_t  egt;     // EG type           (voice data)
    uint8_t  ar;      // EG attack rate    (voice data)          
    uint8_t  dr;      // EG delay  rate    (voice data)
    uint8_t  rr;      // EG release rate   (voice data)
    uint8_t  wavesel; // wave select       (voice data)          
    uint8_t  ksr;     // key scaling rate  (voice data)
    uint8_t  ksl;     // key scaling level (voice data)
    uint8_t  am;      // AM flag           (voice data)
    uint8_t  dam;     // AM depth          (voice data)
    uint8_t  vib;     // vibrato flag      (voice data)
    uint8_t  dvb;     // vibrato depth     (voice data)
    uint8_t  carrf;   // carrier flag 
    uint8_t  op_num;  // Operator number
*/
    uint8_t  egmode;  // EG mode 
    uint8_t  last_egm; // last EG mode                 
    uint8_t  rof;     // KSR offset (0..15)  
    int8_t  gate_q;  // key-ON gate queue
    int8_t  damp_q;  // damper queue
  } op_prop_t;

/* unit propery definition */
typedef struct {
    uint32_t  ph_inc;    // f-number in linear format  
    int32_t   note_f;    // current note number with fraction 
    int32_t   porta_delta; // portamento acc increment (signed)
    int32_t   m_note_f;  // target note number with fraction
    int    waveout;   // waveform output of this unit   
    int    vel_lb;    // MIDI velocity   in log2        
    int    exp_lb;    // MIDI ch vol * MIDI expression in log2        
    int16_t  ks6_0dB;   // Level key scaling value (6.0dB/oct)
    uint8_t  on_cnt;    // multiple keyon counter         
    uint8_t  midi_note; // MIDI note number (0..127)      
    uint8_t  midi_vel;  // MIDI velocity    (0..127)      
    uint8_t  oct;       // effective octave   (0..10)                
    uint8_t  keycode;   // effective keycode  (0..11)
    int8_t  gate;      // key-ON gate flag
    int8_t  damper;    // Hold1/Damper flag
    int8_t  porta_flag;// portamento flag
    uint8_t  hold;      // CC#64 hold1/dumper flag (0..1)
    uint8_t  ksn;       // key scaling number      
    uint8_t  fb;        // modulator feedback depth       
    uint8_t  lfo;       // LFO setting                    
    uint8_t  alg;       // algorithm  
    uint8_t  midi_ch;   // MIDI ch assigned to this unit  
    unit_vparam_t uv;   // unit voice parameter (fb/LFO/alg)
    op_prop_t     op_prop[2]; // property for operator 0, 1   
    int      voice_param_change; // voice parameter change flag
    int      LFO_param_change;   // LFO parameter change flag
#if syn_CV
    int    linCV_oct; // linear CV octave part
    int    linCV_mant;// linear CV mantissa part
#endif
  } unit_prop_t;

/* MIDI channel (part) property */
typedef struct {
     int32_t bend_f;        // pitch bend (b31..b16:int, b15..b0:frac)
     int32_t tune_f;        // ch tune    (b31..b16:int, b15..b0:frac)
     int32_t porta_inc;     // portamento increment
     int32_t vLFO_f;        // vibrato LFO output in fractional note format
    uint16_t aLFO;          // AM (tremolo) LFO output in log2 format
     int16_t waveout;       // wave output                
     int16_t vol_lb;        // MIDI volume     in log2    
     int16_t exp_lb;        // MIDI expression in log2    
     int16_t MD_bend;       // MIDI pitch bend   (-8196..8195)
     int16_t coarse_tune;   // coarse tune      (-64..63)
     int16_t fine_tune;     // fine   tune      (-255..255)
    uint      LFO_ph_acc;    // LFO phase accumulator
    uint      LFO_ph_inc;    // LFO phase acc increment
    uint16_t MD_RPN_ML;     // combined  RPN MSB:LSB    (0x0000..0x7f7f)
    uint16_t MD_NRPN_ML;    // combined NRPN MSB:LSB    (0x0000..0x7f7f)
    uint16_t MD_mod_range;  // MIDI RPN#5 modulation depth range (0..16383)
    uint8_t  MD_bend_range; // MIDI RPN#0 bend range        (0..127) 
    uint8_t  MD_coarse_tune;// MIDI RPN#1 coarse tune MSB   (0..127)
    uint8_t  MD_fine_tune;  // MIDI RPN#2 fine   tune MSB   (0..127)
    uint8_t  MD_progno;     // MIDI program number          (0..127) 
    uint8_t  MD_note;       // MIDI note number             (0..127) 
    uint8_t  MD_velocity;   // MIDI velocity                (0..127) 
    uint8_t  MD_modulation; // MIDI CC#1  modulation        (0..127) 
    uint8_t  MD_porta_time; // MIDI CC#5  portamento time   (0..127)
    uint8_t  MD_volume;     // MIDI CC#7  channel volume    (0..127) 
    uint8_t  MD_pan;        // MIDI CC#10 pan               (0..127) 
    uint8_t  MD_expression; // MIDI CC#11 expression        (0..127) 
    uint8_t  MD_hold1;      // MIDI CC#64 hold1/dumper      (0..127) 
    uint8_t  MD_porta_sw;   // MIDI CC#65 portamento switch (0..127)
    uint8_t  MD_legato;     // MIDI CC#68 legato switch    (0..127)
     int8_t  MD_timbre;     // MIDI CC#71 timbre            (-64..63)
     int8_t  MD_brightness; // MIDI CC#74 brightness        (-64..63)
     int8_t  MD_vib_rate;   // MIDI CC#76 vibrato rate      (-64..63)
     int8_t  MD_vib_depth;  // MIDI CC#77 vibrato depth     (-64..63)
     int8_t  MD_vib_delay;  // MIDI CC#78 vibrato delay     (-64..63)
    uint8_t  MD_porta_src;  // MIDI CC#84 portamento source (0..127)
    uint8_t  MD_rev_send;   // MIDI CC#91 reverb send       (0..127) 
    uint8_t  MD_dent_MSB;   // MIDI CC#6  data entry MSB    (0..127)
    uint8_t  MD_dent_LSB;   // MIDI CC#38 data entry LSB    (0..127)
    uint8_t  LFO_rate;      // 2-bit coded vibrato rate
    uint8_t  LFO_depth;     // 2-bit coded vibrato depth
    uint8_t  last_on_note;  // MIDI note number of most recently ON
    uint8_t  last_off_note; // MIDI note number of most recently OFF
    uint8_t  midi_ch;       // MIDI ch number  
    uint8_t  in_use[1];     // assigned unit number table 
     int8_t  so_tune_ofs[12];  // MIDI scale/octave tuning offset
    op_vparam_t ov[2];      // operator voice parameters
    unit_vparam_t uv;       // unit voice parameters
//    unit_prop_t    patch;    // patch initialization data  
    struct voicedata vdat;   // MA-2 format voice data
  } ch_prop_t; 

/* MIDI master (all channel) property */
typedef struct {
    int32_t   master_tune_f;         // Master tune (b31..b16:int, b15..b0:frac)
    int16_t  master_vol_lb;         // Master volume in log2    
    uint8_t  MD_master_volume;      // MIDI SYSEX master volume      (0..127) 
    uint8_t  MD_master_coarse_tune; // MIDI SYSEX master coarse tune (0..127)
    uint8_t  MD_master_fine_tune;   // MIDI SYSEX master fine   tune (0..127)
  } master_prop_t; 

//#ifdef ADUC7026
/* System configuration structure */
typedef struct {
    int16_t  polyphony;  // 
    int16_t  t1div;      // Timer 1 counter load value
    int16_t  tick1ms;    // tick count for 1 ms
    int16_t  tick4ms;    // tick count for 4 ms
    int16_t  oct_ofs;    // octave offset
    int16_t  note_ofs;   // f-number table offset
    char     fs_asc[3];  // 2-digit ASCII code for fs
  } sys_config_st_t;
//#endif

extern unit_prop_t unit_prop[ABS_MAX_UNITS];

extern ch_prop_t ch_prop[ABS_MAX_PARTS];

extern const sys_config_st_t sys_config[2][MAX_CONFIG];

extern int16_t (*op_ws_p[8])(op_prop_t *o_p);

typedef void (*ch_func1_p)(ch_prop_t *c_p, int32_t arg1);
extern  const ch_func1_p rpn_func_p[MAX_RPN_FUNCS];

extern master_prop_t master_prop;

extern uint8_t sysx_buf[MAXSYSEXLEN];
extern const int8_t centab[128];
extern int16_t eg  (unit_prop_t *u_p, op_prop_t *o_p) ;
//extern int16_t eg2 (unit_prop_t *u_p, op_prop_t *o_p,  int eg_updt) ;
extern int16_t unit(unit_prop_t *u_p);
//extern int16_t op  (op_prop_t *op_p);

extern int16_t op_ws0(op_prop_t *o_p);
extern int16_t op_ws1(op_prop_t *o_p);
extern int16_t op_ws2(op_prop_t *o_p);
extern int16_t op_ws3(op_prop_t *o_p);
extern int16_t op_ws4(op_prop_t *o_p);
extern int16_t op_ws5(op_prop_t *o_p);
extern int16_t op_ws6(op_prop_t *o_p);
extern int16_t op_ws7(op_prop_t *o_p);

extern void print_banner(void);

extern void midi_note_off  (ch_prop_t *c_p, int md_note);
extern void midi_note_on   (ch_prop_t *c_p, int md_note, int md_vel);
extern void calc_am        (ch_prop_t *c_p, unit_prop_t *u_p);
extern void update_LFO     (ch_prop_t *c_p);
extern void calc_porta_time(ch_prop_t *c_p, int ptime);
extern void calc_vibrato   (ch_prop_t *c_p);
extern void calc_tune_f    (ch_prop_t *c_p);
extern void calc_vol_exp   (ch_prop_t *c_p);
extern void calc_pitch     (ch_prop_t *c_p, unit_prop_t *u_p);
extern void pitch_bend     (ch_prop_t *c_p, int32_t bend);
extern void decode_voice   (ch_prop_t *c_p);
extern void copy_voice     (unit_prop_t *u_p);
extern void setup_voice    (ch_prop_t *c_p, int pc);
extern void edit_voice     (ch_prop_t *c_p, int cc,    int val);
extern void pedal_sw       (ch_prop_t *c_p, int pedal, int val);
extern void reset_all_controllers(ch_prop_t *cp);
extern void hard_reset     (void);
extern void all_note_off   (ch_prop_t *c_p);
extern void all_sound_off  (ch_prop_t *c_p);
extern void update_porta   (unit_prop_t *u_p);
extern void calc_keyscaling(unit_prop_t *u_p);
extern void GM_system_on(void);
extern void set_egmode(unit_prop_t *u_p, int egm);

extern void sysx_dec(int len);

extern int  sx_ix;
extern int32_t sx_mfr_id;
extern int  sx_dev_id, sx_sub_id1, sx_sub_id2;

extern int sx_getb(void);

/* SMAF sequencer definitions */

typedef struct {
  const uint8_t *seq;
  const uint8_t inst[4];
        char    *title;
} demo_st;

extern const demo_st demosong[];

/* demo song */
extern const prog_uint8_t arabesq1[];
#ifdef EXTRADEMO
  extern const prog_uint8_t Chopin66[];
  extern const prog_uint8_t invent13[];
#endif

extern int seq_p;
extern void seq_songsel(int song_no);

extern int get_dur  (const uint8_t *sq);
extern int get_event(const uint8_t *sq);

/* Active sensing */
#define MM_ACTSENS 0xfe

/* first byte of event */

#define E1_NOP    0xff
#define E1_MISCEV 0x00

/* 2-byte event code */

#define E2_EXPRESS 0x00
#define E2_PCHBEND 0x01
#define E2_MODURAT 0x02
#define E2_3BYTE   0x03

/* 3-byte event code */

#define E3_PGMCHNG 0x0
#define E3_BANKSEL 0x1
#define E3_OCTSHFT 0x2
#define E3_MODURAT 0x3
#define E3_PCHBEND 0x4
#define E3_VOLUME  0x7
#define E3_PAN     0xa
#define E3_EXPRESS 0xb 

typedef struct {
    int gatetime;   // remaining gatetime
    int octshift;   // octave shift
    int notenumber; // MIDI note number
  } seq_status_t ;

extern seq_status_t seq_st[MAXSEQCH];

#define EG_96dB (1UL<<26)
#define EG_48dB (1UL<<25)
#define EG_60dB (5UL<<23)

#define ABS(x) ((0>x)?(-x):x)

#define LB_to_LIN_s(x,s) (((uint32_t)lb2lin[0x3f & (x)]) << (s - ((uint32_t)(x) >> 6)))
#define LB_to_LIN(x) (((x) >= (EG_96dB>>16)) ? 0 : LB_to_LIN_s(x,16))

#define OPL3MULT(x)  (mult_tab[x])
#define OPL3RATE(x)  ((x)<<2)
#define OPL3LEVEL(x) ((x)<<3)

#if defined(ADUC7026)
  #define LED_OFF (GP4DAT |=  0x00800000) 
  #define LED_ON  (GP4DAT &= ~0x00800000)
  #define CHECKMIDI  (0x01==(0x01 & COMSTA0))
  #define GETMIDI    (COMRX)
  #define PUTMIDI(x) (COMTX = (x))
  #define P0_PWM2H (0x01 << 4)
  #define P0_PWM2L (0x01 << 8)
  #define P0_SIN   (0x02 << 28)
  #define P1_SIN   (0x01)
  #define P1_SOUT  (0x01 << 4)
  #define P2_SOUT  (0x02)
  #define GET_PANEL_KEY (0x01 & GP0DAT)
  extern void arm_port_setup(void);
  #define DELAYMEMSIZE ((6400 - UNIT_RAM_SIZE) >> 1)
  #define SMIDI_SENS_PORT    GP4DAT
  #define SMIDI_SENS_BITMASK 0x08
#elif defined(__v850__)
  #define LED_OFF (PCT.6 = 1, P7H.1 = 1)
  #define LED_ON  (PCT.6 = 0, P7H.1 = 0)
  #define GETMIDI (UA2RIF = 0, UA2RX)
  #define CHECKMIDI (1 == UA2RIF)
  #define PUTMIDI(x) (UA2TX = x)
  #define GET_PANEL_KEY (0x01)
  #define DELAYMEMSIZE (4900)
#elif (defined(STM32F103) | defined(STM32F10X_MD_VL))
  #if defined(__IAR__)
    #define LED_OFF (GPIOC_ODR_bit.ODR6 = 0)
    #define LED_ON  (GPIOC_ODR_bit.ODR6 = 1)
    #define CHECKMIDI (USART1_SR_bit.RXNE)
    #define GETMIDI (USART1_DR)
    #define PUTMIDI(x) (USART1_DR=x)
  #else // #if defind(__IAR__)
    #define LED_L_MACRO(port,bit) (port->ODR =  (port->ODR & ~(1 << bit)))
    #define LED_H_MACRO(port,bit) (port->ODR = ((port->ODR & ~(1 << bit)) | (1 << bit)))
    #if   defined(TGSTCQ)
      #define LED_ON  LED_H_MACRO(GPIOC,6)
      #define LED_OFF LED_L_MACRO(GPIOC,6)
    #elif defined(TGSTBEE)
      #define LED_ON  LED_L_MACRO(GPIOD,4)
      #define LED_OFF LED_H_MACRO(GPIOD,4)
    #elif defined(TGSTBM)
      #define LED_ON  LED_L_MACRO(GPIOA,15)
      #define LED_OFF LED_H_MACRO(GPIOA,15)
    #elif defined(STM32F10X_MD_VL)
      #define LED_ON  LED_H_MACRO(GPIOC,8)
      #define LED_OFF LED_L_MACRO(GPIOC,8)
      #define RCV_LED_ON  LED_H_MACRO(GPIOC,9)
      #define RCV_LED_OFF LED_L_MACRO(GPIOC,9)
      #define USER_KEY (0x01 & GPIOA->IDR)
      #define FORCE_SLOW 1
    #endif
    #if defined(DEMOONLY)
      #define CHECKMIDI (0x00)
    #else
      #define CHECKMIDI ((1<<5) & USART1->SR)
    #endif // #ifdef DEMOONLY
    #define GETMIDI (USART1->DR)
    #define PUTMIDI(x) (USART1->DR=x)
    #define ROMTABLE __attribute__ ((__section__(".rodata.top"), __aligned__(0x200)))
  #endif // #ifdef __IAR__ ... #else
  #define GET_PANEL_KEY (0x01)
  #define DELAYMEMSIZE (4800)
  extern void starm_port_setup( void );
  extern void stm32_port_setup( void );
#else
  #define LED_OFF
  #define LED_ON
#endif

#define DEFAULT_DELAY_ms 100

#if defined(LCD)
  extern void mon_cc_bend_disp(ch_prop_t *c_p, char c1, int MD_2nd, int MD_3rd);
  extern void mon_pc_disp(ch_prop_t *c_p, int MD_2nd);
#endif

#if defined(syn_CV)
  #if defined(__v850__)
    #define PDMWIDTH 42
    #define CALC_PWMVAL(x) ((x - PDMWIDTH)>>1)
  #endif
#endif


#define UNIT_RAM_SIZE (sizeof(unit_prop_t)*ABS_MAX_UNITS + sizeof(ch_prop_t)*ABS_MAX_PARTS)
//#define DELAYMEMSIZE ((6400 - UNIT_RAM_SIZE) >> 1)

#define SW_INC_BIT      0x01
#define SW_DEC_BIT      0x02
#define SW_SFT_BIT      0x04

#define GET_INC_KEY (SW_INC_BIT & sw_bits)
#define GET_DEC_KEY (SW_DEC_BIT & sw_bits)
#define GET_SFT_KEY (SW_SFT_BIT & sw_bits)


#define PART_TO_CH(x) part_to_ch[x]
#define CH_TO_PART(x) ch_to_part[x]


#endif
